@@ -102,7 +102,7 @@ reaster.default <- function(fixed, random, pred, fam, varvar, idvar, root,
     ikeep <- match(names(aout$coefficients), colnames(fixed))
     dropped <- colnames(fixed)[- ikeep]
     nfix.orig <- ncol(fixed)
-    fixed <- fixed[ , ikeep]
+    fixed <- fixed[ , ikeep, drop = FALSE]
     nfix <- ncol(fixed)
     idx.fixed <- seq(1, nfix)
 
