@@ -56,13 +56,15 @@ double aster_mlogl_sat_cond(int nind, int nnode, int *pred, int *fam,
 // R packages calling these functions via the R_GetCCallable mechanism.
 
 #include "mlogl-export.h"
-// apparently this #pragma GCC diagnostic stuff is buggy
-// should eventually work with gcc (>= 6.1) according to gcc bug tracker
-//#pragma GCC diagnostic push
-//#pragma GCC diagnostic ignored "-Wunused-variable"
-//static aster_mlogl_sat_either_funptr foo = aster_mlogl_sat_unco;
-//static aster_mlogl_sat_either_funptr bar = aster_mlogl_sat_cond;
-//#pragma GCC diagnostic pop
+#if defined(__GNUC__) || defined(__clang__)
+static aster_mlogl_sat_either_funptr foo __attribute__ ((unused))
+    = aster_mlogl_sat_unco;
+static aster_mlogl_sat_either_funptr bar __attribute__ ((unused))
+    = aster_mlogl_sat_cond;
+#else
+static aster_mlogl_sat_either_funptr foo = aster_mlogl_sat_unco;
+static aster_mlogl_sat_either_funptr bar = aster_mlogl_sat_cond;
+#endif /* defined(__GNUC__) || defined(__clang__) */
 
 // Except.  We are going to use this typedef below in code
 // that looks like the code that would be in another package
